
<a class="btn btn-navbar" data-toggle="collapse" data-target=".nav-collapse">
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
</a>

<div class="nav-collapse">

    <form action="{% url gnotty_search %}" class="navbar-search pull-right">
        <input required placeholder="Search" type="text" name="q" value="{{ request.GET.q }}">
        {% if request.user.is_authenticated %}
        <a class="btn btn-small logout" href="{{ LOGOUT_URL }}">Logout</a>
        {% endif %}
    </form>

    {% if years %}
    <ul class="nav pull-right">
        <li class="dropdown">
            <a href="#" data-toggle="dropdown">Archives<span class="caret"></span></a>
            <ul class="dropdown-menu">
                {% for year in years %}
                <li><a href="{% url gnotty_year year %}">{{ year }}</a></li>
                {% endfor %}
            </ul>
        </li>
        <li class="divider-vertical"></li>
    </ul>
    {% endif %}

</div>
