{% load gnotty_tags %}<!doctype html>
<html lang="en">

<head>
<meta http-equiv="Content-type" content="text/html; charset=utf-8">
<meta name="viewport" content="width=device-width, height=device-height, initial-scale=1.0, user-scalable=0, minimum-scale=1.0, maximum-scale=1.0">
<title>{{ IRC_CHANNEL }}</title>
<link rel="stylesheet" href="{{ STATIC_URL }}css/bootstrap.spacelab.css">
<link rel="stylesheet" href="{{ STATIC_URL }}css/bootstrap.responsive.css">
<link rel="stylesheet" href="{{ STATIC_URL }}css/gnotty.css">
<script src="{{ STATIC_URL }}js/jquery.min.js"></script>
<script src="{{ STATIC_URL }}js/bootstrap.min.js"></script>
<script src="{{ STATIC_URL }}js/json2.js"></script>
<script src="{{ STATIC_URL }}js/socket.io.min.js"></script>
<script src="{{ STATIC_URL }}js/jquery.tmpl.min.js"></script>
<script src="{{ STATIC_URL }}js/urlize.js"></script>
<script src="{{ STATIC_URL }}js/gnotty.js"></script>
<!--[if lt IE 9]>
<script src="//html5shiv.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
{% block extrahead %}{% endblock %}
</head>

<body>

<div class="navbar navbar-fixed-top">
<div class="navbar-inner">
<div class="container">
    <a class="brand" href="{% url gnotty_chat %}">{{ IRC_CHANNEL }}</a>
    {% gnotty_nav %}
</div>
</div>
</div>

<div class="container">
{% block content %}{% endblock %}
</div>

<footer>
<form class="container form-inline chat" action="{% url gnotty_chat %}">
<div class="row">
<div class="span9 control-group">
<div class="controls">
    <input type="text" class="input-xlarge nick" id="input" placeholder="nickname" name="nickname" autocomplete="off" maxlength="{{ MAX_MESSAGE_LENGTH }}">
    <input type="submit" class="btn btn-primary" id="submit" value="Join">
    <input type="button" class="btn" id="leave" value="Leave">
</div>
</div>
<p class="span3 powered">Powered by <a href="{{ GNOTTY_PROJECT_URL }}">{{ GNOTTY_VERSION_STRING }}</a></p>
</div>
</form>
</footer>

</body>
</html>
