
body {margin:70px 0 65px 0;}
.navbar-fixed-top {position:fixed !important;}
.navbar-fixed-top, .navbar-inner {padding:0 !important;}

footer {position:fixed; left:0; bottom:0; background:#ddd; width:100%;
        border-top:1px solid #999; padding:0 20px;}

table {margin-top:-15px;}
th {border-top:30px solid #fff !important;}
.calendar th, .calendar td {text-align:center;}
.calendar .not-in-month, .calendar .not-in-month a {color:#ddd;}
.calendar a {font-weight:bold; text-decoration:underline;}

.main-chat {display:none;}
.main-chat a {text-decoration:underline;}
.message {width:100%;}
#messages td, .nickname {border:0;}
#messages .time {color:#888; font-size:12px;}
#messages .nickname {text-align:right;}
.message-highlighted td {background:#ffc !important;}

#nicknames {margin-bottom:0;}
.nicknames-row {position:fixed; top:60px; bottom:90px; overflow-y:scroll; z-index:-1;}
.nicknames-wrap {margin-left:-50px;}

.dropdown-menu a {text-align:right;}
.chat {margin-top:5px; margin-bottom:0; padding:15px 0;}
.chat input {margin-right:10px;}
#leave {display: none;}

.loading {margin-top:0; top:40%;}
.progress {margin-bottom:0;}
.bar {width:1px;}
.modal-backdrop {background:#fff;}

.logout {margin:-5px 0 0 5px !important;}

.powered {padding:10px 0; margin:0 10px; text-align:right;}

@media (max-width:979px) {
    .nick, #password {max-width:40%;}
}

@media (max-width: 767px) {
    body {margin-top:30px;}
    .nick, #password {max-width:32%;}
    .msg {max-width:55%;}
    .nicknames-row, .navbar-fixed-top, .submit-joined, .powered {display:none !important;}
}
